<?php
namespace Controller;

use Util\Utilities;
use Service\DeleteCascadeService;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Service\Traits\TResponse;

class GridStyle extends Crud {
	
	use TResponse;

	protected $dataSourceName = 'gridStyle';
	
	public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService) {
		parent::__construct($organizationMongoManager, $deleteCascadeService);
	}

	public function findNext(DTO\Request $request, DTO\Response $response) {
        try{
            $params = $request->getParameters();

            Utilities::convertDynamicFilterToArray($params);
            if (array_key_exists("indicator", $params)) 
                $params["dynamic_filter"]["indicator"] = [["relation"=>  "AND", "operation" => "EQUALS", "value"=> $params["indicator"]]];

            $data = $this->mongoManager->findNext($this->dataSourceName, $params, false);
            $total = $this->countNext($params, false);
            $dataSet = Utilities::parseDataSourceZeedhiNext($data, $total);
            $dataSet['pagination']['page'] = array_key_exists('page', $params) ? intval($params['page']) : 1;
            $dataSet['pagination']['limit'] = array_key_exists('limit', $params) ? intval($params['limit']) : 99999;
            $this->sendResponse($dataSet, 200);
        } catch(\Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getChartViewStyle($indicator){
        $decodeBase64 = false;
        $params = array(
            'page' => 1,
            'limit' => 999999,
            'dynamic_filter' => array(
                'indicator' => array(
                    array(
                        'relation' => 'AND',
                        'operation' => 'EQUALS',
                        'value' => $indicator,
                    ),
                )
            ),
        );
        $data = $this->mongoManager->findNext($this->dataSourceName,$params,$decodeBase64);
        $styleNames = array();
        foreach ($data as $row) {
            $styleNames[] = $row['styleName'];
        }
        return $styleNames;
    }

	public function countNext($params, $decode = true) {
        return $this->mongoManager->countNext($this->dataSourceName, $params, $decode);
    }
}