import { Http, Metadata, IDictionary } from '@zeedhi/core';
import { Col, Form, ModalService, Card, LoadingService, SelectTree } from '@zeedhi/common';
import { Utilities } from '@/Services/Utilities';
import { CustomAlertService } from '@/Services/CustomAlertService';
import { SkeletonMetadata } from '@/Services/SkeletonMetadata';
import { ChartViewService } from '../Services/ChartViewService';
import { ZmartChartsApex } from '../Services/Chats/ZmartChartsApex';

export class ChartViewController {
    public filterModal: any = {};
    private chartComponent: any = null;
    public viewId: any = null;
    public drillDownFilters: any = {};
    private widget: any = null;
    private chart: any = null;
    public loadingTemplate = {
            "name": "result",
            "component": "ZdImage",
            'height': "100%",
            "src": "../img/loading3.gif"
        };
    public drillDownViewStack: any = [];
    public drillDownFiltersList: any = {};

    public async apexChartOnCreated({ component }:any) {
        const isDrillDown = !!component.parent.parent.fullscreen;
        const urlParams = new URLSearchParams(window.location.search);
        let viewId = urlParams.get('name') ? urlParams.get('name') : '';
        let filters = {};

        if (localStorage.getItem('drillDownView')) {
            viewId = localStorage.getItem('drillDownView');
            localStorage.setItem('drillDownView', "");
        }
        if (localStorage.getItem('drillDownFilter')) {
            filters = JSON.parse(localStorage.getItem('drillDownFilter') || "{}");
            localStorage.setItem('drillDownFilter', "");
        }

        if (typeof viewId === 'string') {
            this.buildSkeletonChart(component);
            this.chartComponent = component;
            this.viewId = viewId;
            await this.buildWidgetByViewId(component, viewId, filters);
        }
    }

    public buildSkeletonChart(component: any) {
        component.children = [
            SkeletonMetadata.skeletonIndicator().children[0]
        ];
    }

    public async cardTitleOnMounted({ component }:any) {
        const urlParams = new URLSearchParams(window.location.search);
        const viewId = urlParams.get('name') ? urlParams.get('name') : '';
        if (viewId) {
            let element: Element | null = document.getElementById(viewId);
            while (element) {
                if (element.className === 'tek-card-title-text') {
                    element.setAttribute('style', component.cssStyle);
                    break;
                }
                element = element.children && element.children[0] ? element.children[0] : null;
            }
        }
    }

    public replaceAnchorElement({element}: any) {
        const anchorElement = element.querySelector("a.tek-card-title-no-router");
        if (anchorElement) {
            const newElement = document.createElement('p');
            for (let i = 0; i < anchorElement.attributes.length; i += 1) {
                const attr = anchorElement.attributes[i];
                newElement.setAttribute(attr.name, attr.value);
            }
            newElement.innerHTML = anchorElement.innerHTML;
            newElement.classList.add("zd-ma-0");
            anchorElement.replaceWith(newElement);
        }
    }

    public updateHeaderText(clearText: Boolean, textList ?: any, chartViewComponent ?: any) {
        [
            chartViewComponent.parent.parent.children[0].leftSlot[0],
            chartViewComponent.parent.parent.children[0].leftSlot[2],
            chartViewComponent.parent.parent.children[0].leftSlot[4]
        ].forEach((component: any, i: any) => {
            component.text = clearText ? "" : textList[i];
        });
        [
            chartViewComponent.parent.parent.children[0].leftSlot[1],
            chartViewComponent.parent.parent.children[0].leftSlot[3]
        ].forEach((component: any, i: any) => {
            component.isVisible = !clearText && (!!textList[i + 1] && typeof textList[i + 1] === "string");
        });
    }

    private getPeriodString(period: any) {
        if (Array.isArray(period) && period[0]) {
            period = period[0];
        }
        return (period.start + " até " + period.end);
    }

    public async buildWidgetByViewId(component: any, viewId: string, filters: any, structureId ?: any, structureName ?: any) {
        if (!filters) {
            filters = {};
        }
        if (!structureId) {
            structureId = "";
        }
        if (!structureName) {
            structureName = "";
        }
        const params = {
            "requestType": "Row",
            "row": {
                "drillDownFilters": filters,
                "serieColors": 0,
                "structureId": structureId,
                "structureName": structureName,
                "viewId": viewId,
                "period": filters.period ? filters.period : null,
                "dimension": filters.dimension ? filters.dimension : null,
                "facts": filters.facts ? filters.facts : null,
                "isChartView": true,
            }
        };
        const response = await Http.post("/buildWidgetByViewIdNext", params);
        const widget = response.data.dataset.widget;
        if (response.data.dataset.error && response.data.dataset.error.length > 0) {
            Utilities.widgetErrorMessageAlerts(response.data.dataset.error);
        }
        this.widget = widget;
        this.drillDownFilters = filters;
        component.children = [];
        if (widget[0].name === 'notFound') {
            setTimeout(function () {
                component.children = widget;
            }, 0);
        } else {
            if (viewId !== this.viewId) {
                this.viewId = viewId;
            }
            this.chart = ZmartChartsApex.build(widget, this, component);

            const structureText = (structureName || widget[0].widget.parameters.structureName[0]);
            if (!filters.period && widget[0] && widget[0].widget && widget[0].widget.parameters && widget[0].widget.parameters.period && widget[0].widget.parameters.period && widget[0].widget.parameters.period.start && widget[0].widget.parameters.period.end) {
                filters.period = widget[0].widget.parameters.period;
            }
            const periodText = filters.period ? this.getPeriodString(filters.period) : "Todo o período";
            const drillText = this.drillDownFilters ? (Object.values(this.drillDownFilters).filter((value: any) => typeof value === 'string')) : "";
            this.updateHeaderText(false, [structureText, periodText, (typeof drillText === 'object' ? drillText.join(" - ") : drillText)], component);
            if (filters) {
                if (filters.drillDownFilters && Object.values(filters.drillDownFilters).length > 0) {
                    filters = {...filters, ...filters.drillDownFilters};
                }
                this.chart.chartInstance.period = filters.period;
                if (Array.isArray(filters.period) && filters.period[0]) {
                    this.chart.chartInstance.period = filters.period;
                }
                const drillFiltersFieldValue: any = [];
                Object.entries(filters).forEach(function ([key, value]: any) {
                    if (!["period", "drillDownFilters", "facts", "dimension"].includes(key)) {
                        drillFiltersFieldValue.push({ name: key, value });
                    }
                });
                this.chart.chartInstance.drillDownFilters = drillFiltersFieldValue;
            }
            const widgetChats = this.chart.getCharts();
            component.children = widgetChats;
        }
    }

    protected evalCondition(condition: String, variables: any) {
        condition = condition.replaceAll('row.', '');
        const operators: any = {
          '==': (a: any, b: any) => a === b,
          '!=': (a: any, b: any) => a !== b,
          '<': (a: any, b: any) => a < b,
          '>': (a: any, b: any) => a > b,
          '<=': (a: any, b: any) => a <= b,
          '>=': (a: any, b: any) => a >= b,
        };

        const conditions = condition.split(/&&|\|\|/);
        const conditionComparators: any = condition.match(/(\|\||&&)/g);

        const results = conditions.map((singleCondition: any) => {
            if (singleCondition === "true") {
                return true;
            }
            if (singleCondition === "false") {
                return false;
            }
            const parts = singleCondition.match(/\S+/g);

            if (parts.length !== 3) {
                console.error("Condição inválida: ", singleCondition);
                return false;
            }

            const [leftOperand, operator, rightOperand] = parts;
            let leftValue = variables[leftOperand];
            if (leftValue === undefined) {
                leftValue = parseInt(leftOperand, 10);
            }
            let rightValue = variables[rightOperand];
            if (rightValue === undefined) {
                rightValue = parseInt(rightOperand, 10);
            }
            if (leftValue === undefined || rightValue === undefined) {
                console.error("Condition error: ", singleCondition);
                return false;
            }
            return operators[operator](leftValue, rightValue);
        });
        if (conditionComparators) {
            while (conditionComparators.length > 0) {
                const comparator = conditionComparators.shift();
                const a = results.shift();
                const b = results.shift();
                if (comparator === "||") {
                    results.unshift(a || b);
                }
                if (comparator === "&&") {
                    results.unshift(a && b);
                }
            }
        }
        return results[0];
    }

    protected cssStringToJSON(cssString: any) {
        const cssRegex = /([^:;]+):([^;]+);/g;
        const cssObject : any = {};
        let match = cssRegex.exec(cssString);
        while (match !== null) {
          const propertyName = match[1].trim();
          const propertyValue = match[2].trim();

          cssObject[propertyName] = propertyValue;
          match = cssRegex.exec(cssString);
        }
        return cssObject;
      }

    public applyGridStylesCrosstab(crosstab: any, cell: any) {
        const row: any = {};
        const gridStyles = this.chart.chartInstance.zmartCharts.chartMetadata[0].children[1].widget.gridStyle;
        const column = this.chart.chartInstance.zmartCharts.chartMetadata[0].children[1].widget.yKeys;
        cell.row.forEach(function (r: any) {
            row[r] = cell.value;
        });
        return this.applyStyles(gridStyles, column, row);
    }

    public applyGridStyles({row, column}: IDictionary<any>) {
        let gridStylesField = {};
        if (this.chart) {
            const gridStyles = this.chart.chartInstance.zmartCharts.chartMetadata[0].widget.gridStyle;
            gridStylesField = this.applyStyles(gridStyles, column, row);
        }
        return gridStylesField;
    }

    protected columnCompare(column: any, styles: any) {
        if (Array.isArray(column)) {
            return column.some((element: any) => styles.includes(element));
        }
        return styles.includes(column);
    }

    protected applyStyles(gridStyles: any, column: any, row: any) {
        if (gridStyles) {
            let cssStyle: any = {};
            gridStyles.forEach((style: any) => {
                if (this.columnCompare(column, style.styleFields) && this.evalCondition(style.condition, row)) {
                    const code = style.css.cssCode.match(/{([^}]*)}/);
                    if (code) {
                        cssStyle = {...cssStyle, ...this.cssStringToJSON(code[1])};
                    }
                }
            });
            return cssStyle;
        }
        return {};
    }

    public backButton() {
        this.chart.goBackDrill();
    }

    public openFilterModal() {
        this.createFilterModal();
    }

    public async createFilterModal() {
        LoadingService.show();
        const modal = await this.chart.chartInstance.getFilterModal();
        this.filterModal = ModalService.create(modal);
        this.filterModal.show();
        LoadingService.hide();
    }

    public closeFilterModal() {
        this.filterModal.hide();
        this.filterModal?.destroy();
    }

    public async applyFilter() {
        this.updateHeaderText(true, [], Metadata.getInstance<Card>("card-view-chart"));
        const form = Metadata.getInstance<Form>('filterForm');
        const filters: any = {};
        let structureId: any = null;
        let structureName = null;
        if (!!form.value.period && !!form.value.period.length) {
            const dateStart = form.value.period[0];
            const dateEnd = form.value.period[1];
            filters.period = {
                "start": dateStart,
                "end": dateEnd
            };
        }

        if (form.value.drillDownFilters) {
            filters.drillDownFilters = {};
            form.value.drillDownFilters.reduce((acc: any, obj: any) => {
                acc[obj.name] = obj.value;
                return acc;
            }, filters.drillDownFilters);
        }

        if (form.value.structure_label !== undefined) {
            structureId = form.value.structure_label;
            const selectComponent = Metadata.getInstance<SelectTree>("structure_label");
            if (selectComponent) {
                const structureRow: any = selectComponent?.datasource?.data.find((data: any) => data._id === structureId);
                structureName = structureRow.name;
            }
        }

        if (form.value.dimensionFilter) {
            filters.dimension = form.value.dimensionFilter;
        }

        if (form.value.factsFilter) {
            filters.facts = form.value.factsFilter;
        }

        this.closeFilterModal();
        this.buildSkeletonChart(this.chartComponent);
        await this.buildWidgetByViewId(this.chartComponent, this.viewId, filters, structureId, structureName);
    }

    public static openEditView() {
        const colViewEdit = Metadata.getInstance<Col>('col-view-edit');
        colViewEdit.isVisible = true;
        colViewEdit.cols = 4;
        colViewEdit.sm = 4;
        colViewEdit.md = 4;
        colViewEdit.lg = 4;
        const colViewChart = Metadata.getInstance<Col>('col-view-chart');
        colViewChart.cols = 8;
        colViewChart.sm = 8;
        colViewChart.md = 8;
        colViewChart.lg = 8;
        ChartViewController.createFieldsEditView().then();
    }

    public static closeEditView() {
        const colViewEdit = Metadata.getInstance<Col>('col-view-edit');
        colViewEdit.isVisible = false;
        const colViewChart = Metadata.getInstance<Col>('col-view-chart');
        colViewChart.cols = 12;
        colViewChart.sm = 12;
        colViewChart.md = 12;
        colViewChart.lg = 12;
    }

    public static async createFieldsEditView() {
        const params = {
            "requestType": "FilterData",
            "filter": {}
        };
        const response = await Http.post("/viewProperty", params);
        if (response.data.dataset.viewProperty) {
            const colViewEdit = Metadata.getInstance<Col>('form-view-prop-edit');
            response.data.dataset.viewProperty.forEach(function (viewProperty: any) {
                if (viewProperty) {
                    viewProperty.component = "ZdTextInput";
                }
            });
        }
    }

    public static setFieldsEditView(typeView: any) {
        const colViewEdit = Metadata.getInstance<Col>('form-view-prop-edit');
        colViewEdit.children.forEach(function (field: any) {
            if (typeView.properties.indexOf(field._id) !== -1) {
                field.isVisible = true;
            } else {
                field.isVisible = false;
            }
        });
    }

    private getValueFromRow(row: any) {
        return row.infinity ? Infinity : parseFloat(row.changeValue);
    }

    public periodCompareChangeActionGetVisibility({row}: any) {
        return !!row.change;
    }

    public periodCompareChangeActionGetColor({row}: any) {
        const value = this.getValueFromRow(row);
        if (value < 0) {
            return 'error';
        }
        if (value > 0) {
            return 'success';
        }
        return 'grey';
    }

    public periodCompareChangeActionGetIconName({row}: any) {
        const value = this.getValueFromRow(row);
        if (value < 0) {
            return 'mdi-chevron-down';
        }
        if (value > 0) {
            return 'mdi-chevron-up';
        }
        return '';
    }

    public periodCompareChangeActionGetLabel({row}: any) {
        const value = this.getValueFromRow(row);
        return (value === Infinity ? '∞' : Math.abs(value)) + '%';
    }
}
