import { Metadata, FormatterParserProvider, Loader } from '@zeedhi/core';
import { Modal, ModalService, Form, DialogService, LoadingService, Select, SelectMultiple, DateRange, Col, Container } from '@zeedhi/common';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';
import { Utilities } from '@/Services/Utilities';
import { DataSourceService } from '@/Services/DataSourceService';
import { Highlight } from '@zeedhi/zd-highlight-common';
import Prism from 'prismjs';
import 'prismjs/components/prism-markup-templating.min';
import 'prismjs/components/prism-php.min';

FormatterParserProvider.registerFormatter('PeriodoFormatter', (periodObject: any) => "De " + periodObject.start + " até " + periodObject.end);

export class DataSourceController {
    public service = new DataSourceService();
    // Controle do Master Detail
    public masterDetailControl: any = {
        grid: true,
        detail: false,
        apiDetail: false
    };
    public showMasterDetail(option: string) {
        Object.keys(this.masterDetailControl).forEach((key: string) => { this.masterDetailControl[key] = key === option; });
    }
    public get isGridVisible() {
        return this.masterDetailControl.grid;
    }
    public get isDetailVisible() {
        return this.masterDetailControl.detail;
    }
    public get isAPITabsVisible() {
        return this.masterDetailControl.apiDetail;

    }
    public showGrid() {
        this.showMasterDetail("grid");
    }
    public showDetail() {
        this.showMasterDetail("detail");
    }
    public showAPIDetail() {
        this.showMasterDetail("apiDetail");
    }

    //  Referente ao componente Highlight
    public get addDatasourceQuery() {
        const queries = Metadata.getInstances('query');
        return queries.find((query: any) => query.parent.name === 'modalDataSourceForm');
    }
    public get editDatasourceQuery() {
        const queries = Metadata.getInstances('query');
        return queries.find((query: any) => query.parent.name === 'dataSourceForm');
    }
    public loadLanguage = this.loadLanguageAux.bind(this);
    public loadLanguageAux() {
        const query = this.isGridVisible ? this.addDatasourceQuery : this.editDatasourceQuery;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return "";
    }
    public loadLanguageExpression(params: any) {
        const { code, language } = Metadata.getInstance<Highlight>('expression');
        return Prism.highlight(code, Prism.languages[language], language);
    }
    // Métodos de Dialog e Modal
    public showDeleteDialog(params: any) {
        try {
            Utilities.genericBeforeDeleteTekGrid(params, "/dataSource/remove", "datasource-grid");
        } catch (e) {
            console.log(e);
        }
    }

    // TRECHO DE CODIGO RELACIONADO AO MASTERDETAIL, GRID E FORM DO DATASOURCE
    private modal!: Modal;
    public closeDetails() {
        if (this.isDetailVisible) {
            this.showGrid();
        } else {
            this.modal.hide();
        }
    }
    public async createAddModal() {
        this.modal = await ModalService.createFromJson('dataSource/DataSource/AddModal', true);
    }

    public datasourceData: any;
    public detailTitleText: string = '';
    public switchToDetail({ component, row }: any) {
        if (component.editing === true) return;
        this.datasourceData = row;
        this.detailTitleText = this.datasourceData?.name || "Fonte de Dados";
        this.showDetail();
        this.fillDataSourceForm({ row });
        const columnGrid = Metadata.getInstances("gridColumns");
        if (columnGrid.length > 0 && columnGrid[0]) {
            this.columnGridChange({ component: columnGrid[0] });
        }
        Loader.getInstance("DSIndexesController").updateIndexesFilter();
        Loader.getInstance("DSImportScheduleController").importSchedulingUpdateForm();
        Loader.getInstance("DSImportHistoryController").updateImportHistoryFilter();
        Loader.getInstance("DSAPIController").updateAPIFilter();
        Loader.getInstance("DSDataVisualization").updateDataVisualization();
        Loader.getInstance("DSLogController").updateLog();
    }
    public async openModal({ component, row }: any) {
        this.modal?.destroy();
        await this.createAddModal();
        this.datasourceData = {joinDataSource: "N", hasDataMining: "S", willUpdateIndicators: "N", isScheduleActive: "S"};
        this.modal.show();
    }
    public columnGridChange({ component }: any) {
        const columns = this.datasourceData.columns || [];
        component.datasource.updateData(columns);
    }

    //  Metodos do Form
    public fillDataSourceForm({ row }: any) {
        if (!row) {
            return;
        }
        const values = row || this.datasourceData;
        const form = Metadata.getInstance<Form>("dataSourceForm");
        const query = this.isGridVisible ? this.addDatasourceQuery : this.editDatasourceQuery;
        const accessProfiles = form.getChildInstance<SelectMultiple>("accessProfiles");
        form.value = values;
        query.code = values.query_view || values.query || "";
        accessProfiles.setValue(values.accessProfiles || []);
        if (localStorage.getItem('isTeknisa') === 'false' && row.IS_TEKNISA === 'Y') {
            form.childrenInstances[7].isVisible = false;
            form.childrenInstances.forEach((component) => {
                component.readonly = true;
            });
        } else {
            form.childrenInstances[7].isVisible = true;
            form.childrenInstances.forEach((component) => {
                component.readonly = false;
            });
        }
    }

    public saveDatasourceFormConfirmationDialog() {
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: 'Salvar fonte de dados',
            text: 'Deseja salvar a fonte de dados?',
            buttons: [
              { name: 'no', component: 'ZdButton', label: 'NO', events: {click: () => DialogService.hide()}, outline: true},
              { name: 'yes', component: 'ZdButton', label: 'YES', events: {click: () => this.saveDatasourceFormConfirm()}},
            ],
          });
    }

    private saveDatasourceFormConfirm() {
        DialogService.hide();
        this.saveDatasourceForm();
    }

    public async saveDatasourceForm() {
        const form = this.isGridVisible ? Metadata.getInstance<Form>("modalDataSourceForm") : Metadata.getInstance<Form>("dataSourceForm");
        const query = this.isGridVisible ? this.addDatasourceQuery : this.editDatasourceQuery;
        const values = form.value;
        const connection = values.connection;
        const queryCode = query.code ? this.service.parseQueryCode(query.code) : "";
        if (!form.validate() || !queryCode.trim()) {
            if (!queryCode.trim()) {
                CustomAlertService.errorAlert("O campo de código é obrigatório e deve ser preenchido");
            }
            return;
        }
        const data = {
            ...values,
            changed: true,
            columns: this.datasourceData.columns || [],
            connection: connection?._id,
            connectionName: connection?.name,
            room: connection?.room,
            createdBy: this.datasourceData.userId,
            importStatus: this.datasourceData.importStatus || "",
            originDataSources: this.datasourceData.originDataSources || "",
            query: queryCode,
            query_view: query.code || "",
            relations: this.datasourceData.relations || [],
            _queryChanged: this.datasourceData._id ? !(this.datasourceData.query === queryCode) : true,
            _id: this.datasourceData?._id
        };
        try {
            const res = await this.service.saveDataSource(data, this.user);
            console.log(res);
            CustomAlertService.saveSuccessAlert();
            if (res.data.messages?.length > 0) {
                const message = res.data.messages[0].message;
                DialogService.show({
                  name: 'warning-dialog',
                  type: 'error',
                  title: ' ',
                  text: message
                });
            }
            this.reloadGrid();
            this.closeDetails();
        } catch (err) {
            LoadingService.hide();
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }

    public user: any;
    public async updateLoggedUserInfo() {
        const info = await Utilities.getLoggedUser();
        this.user = info;
    }

    public handleConnectionResponse({ component }: any) {
        const info = component?.currentRow;
        if (info?.WALLET_NRORG === undefined) {
            return;
        }
        const {data, WALLET_NRORG, pagination} = info;
        component.data = data;
        component.total = pagination.total;
        component.limit = pagination.limit;
        component.page = pagination.page;
        component.currentRow = data;
    }
    public reloadGrid() {
        const grid = Metadata.getInstance<TekGrid>("datasource-grid");
        grid.reload();
    }
    public async saveInlineEdit(params: any) {
        Utilities.genericBeforeSaveTekGrid(params, "dataSource/save", "datasource-grid");
    }

    public adjustFilter({ component }: any) {
        const datasource = component.datasource;
        const filter = datasource.dynamicFilter;
        Object.entries(filter).forEach(([k, filters]: any) => {
            filter[k] = filters.filter(({value}: any) => !(value instanceof Array && value.length === 0));
            if (filter[k].length === 0) {
                delete filter[k];
            }
        });
    }
}
