import { Metadata, Http, Router, FormatterParserProvider } from '@zeedhi/core';
import { Modal, ModalService, TreeGrid, Form, LoadingService, AlertService, DialogService } from '@zeedhi/common';
import { CustomAlertService } from '@/Services/CustomAlertService';

FormatterParserProvider.registerFormatter('column_SubstitutoAction', ({ row }: any) => {
    const type: string = row.type;
    const options: any = {
        "D": "Preview",
        "I": "View"
    };
    return options[type];
});

export class IndicatorsDatabaseController {
    public routesRelation: any = {
        "viewPreview": "/chartView",
        "dashboardPreview": "/semTela"
    };
    public handleViewRedirect = ({ row }: any) => this.routesRelation.viewPreview + "?Name=" + row._id;
    public handleDashboardRedirect({ row }: any) {
        return row.type === "D" ? this.routesRelation.dashboardPreview + "?Name=" + row._id : undefined;
    }
    public modalsPath: string = "indicatorsDatabase/";
    public modalRelation: any = {
        "detail": "DetailModal/DetailModal",
        "indicator": "IndicatorModal/IndicatorModal",
        "import": "ImportModal/ImportModal",
        "filter": "FilterModal/FilterModal",
        "connection": "ConnectionModal/ConnectionModal"
    };
    public gridName = "indicatorsGrid";
    //  Metodos de dialog
    public showErrorDialog(title: string, text: string) {
        DialogService.show({
            name: 'error-dialog',
            type: 'error',
            title,
            text
        });
    }
    public showSuccessDialog(title: string, text: string) {
        DialogService.show({
            name: 'success-dialog',
            type: 'success',
            title,
            text
        });
    }
    //  Metodos de Modal
    private detailModal!: Modal;
    private indicatorModal!: Modal;
    public createModal(type: any) {
        const path: any = this.modalsPath + this.modalRelation[type];
        return ModalService.createFromJson(path, true);
    }
    //  IndicatorModal
    public indicatorOnMounted({ component }: any) {
        const data = this.modalData;
        const form = component;
        const formValue = form.value;
        Object.keys(formValue).forEach((key) => { formValue[key] = data[key]; });
    }
    public visaoOnMounted({ component }: any) {
        const data = this.modalData;
        const datasource = component.datasource;
        datasource.setFilter({"indicator": data._id});
    }
    //  DetailForm
    protected modalData: any;
    public get type() {
        return this.modalData.type;
    }
    public get modalTitle() {
        const dict: any = {
            "M": "Menu",
            "I": "Indicador",
            "D": "Dashboard"
        };
        return dict[this.type];
    }
    public get modalOrderText() {
        const dict: any = {
             "M": "Menu",
            "I": "Menu",
            "D": "Dashboard"
        };
        return "Ordem no " + dict[this.type];
    }
    public get isDashboard() {
        return this.type === "D";
    }
    public async formOnMounted({ component }: any) {
        const form = component;
        const formValue = form.value;
        const data = this.modalData;
        formValue.name = data.name;
        formValue.order = data.order;
        formValue.dashboardTemplate = data.dashboardTemplate;
    }
    public openModalOnView: boolean = false;
    public async openModal({ event, row}: any) {
        //  Checa se o evento foi de fato expandir a linha ao invés de click na mesma
        if (event.target.type === "button" || !row.type) {
            return;
        }
        let modal;
        if (row.type === "I") {
            this.indicatorModal?.destroy();
            this.indicatorModal = await this.createModal("indicator");
            modal = this.indicatorModal;
            if (this.openModalOnView) {
                const tabs = modal.children[1];
                tabs.activeTab = 1;
                this.openModalOnView = false;
            }
        } else {
            this.detailModal?.destroy();
            this.detailModal = await this.createModal("detail");
            modal = this.detailModal;
        }
        this.modalData = row;
        modal.show();
    }
    public closeIndicatorModal() {
        this.indicatorModal?.destroy();
    }
    public closeDetailModal() {
        this.detailModal?.destroy();
    }
    public correctFilter({ component }: any) {
        const datasource = component.datasource;
        const filter = datasource.filter;
        if (filter.label) {
            filter.label = "%" + filter.label + "%";
        }
    }
    public adjustFilterFields({ component }: any) {
        const datasource = component.datasource;
        const filter = datasource.filter;
        if (filter.label && filter.label.startsWith('%') && filter.label.endsWith('%')) {
            filter.label = filter.label.slice(1, -1);
        }
    }
    //  Grid Methods
    public getConditionalsClassAlreadyImported({row}: any) {
        return row.alreadyImported ? "indicatorsDatabaseAlreadyImported" : "";
    }
    public getConditionalsHasAction({row}: any) {
        return ["I", "D"].includes(row.type);
    }
    public async reload() {
        const grid = Metadata.getInstance<TreeGrid>(this.gridName);
        const res = await grid.buildTree();
    }
    public async handleClick({ event, row }: any) {
        event.stopPropagation();
        if (row.type === "I") {
            this.openModalOnView = true;
            await this.openModal({ event, row });
        }
    }
    //  Import Modal
    public importModal!: Modal;
    public get noRowSelected() {
        const grid = Metadata.getInstance<TreeGrid>(this.gridName);
        return grid.selectedRows.length <= 0;
    }
    public async openImportModal() {
        this.importModal?.destroy();
        const res = await Http.get("checkCreateNewConnection");
        const data = res.data.data;
        if (data.hasConnection || data.idOrgSaas) {
            this.importModal = await this.createModal("import");
            const grid = Metadata.getInstance<TreeGrid>(this.gridName);
            this.modalData = grid.selectedRows;
            this.importModal.show();
        } else {
            this.openConnectionModal();
        }
    }
    public async importAccessProfile() {
        const form = this.importModal.getChildInstance<Form>("importForm");
        if (!form.validate()) {
            return;
        }
        const values = form.value;
        const rows = this.modalData;
        const accessProfiles = values.accessProfiles;
        const res = await Http.post("importDefault", {
            "requestType": "Row",
            "row": {
                "accessProfiles": accessProfiles || [],
                "rows": rows
            }
        });
        this.importModal.hide();
        this.reload();
        CustomAlertService.successAlert("Importado com sucesso!");
    }
    public closeImportModal() {
        this.importModal?.hide();
    }
    //  Main Filter
    public filterModal!: Modal;
    public async openFilterModal() {
        this.filterModal?.destroy();
        this.filterModal = await this.createModal("filter");
        this.filterModal.show();
    }
    public closeMainFilter() {
        this.filterModal?.hide();
    }
    public async applyMainFilter() {
        const grid = Metadata.getInstance<TreeGrid>(this.gridName);
        const datasource = grid.datasource;
        const form: Form = this.filterModal.getChildInstance("indicatorsGrid-filter-form");
        const values = form.value;
        const filters = {
            "name": values.name ? "%" + values.name + "%" : undefined,
            "type": values.type.length > 0 ? values.type : undefined
        };
        // Se houver algum filtro com valor
        if (Object.values(filters).reduce((el, filtro) => filtro || el)) {
            await datasource.setFilter({...filters, "parentTreeId": -1 });
        } else {
            await datasource.setFilter({});
        }
        this.closeMainFilter();
    }
    public filterOnMounted({ component }: any) {
        const filter = Metadata.getInstance<TreeGrid>(this.gridName).datasource.filter;
        const form = component;
        const formValue = form.value;
        formValue.name = filter.name ? filter.name.slice(1, -1) : "";
        form.getChildInstance('type').value = filter.type;
    }
    //  Connection Modal
    public connectionModal!: Modal;
    public async openConnectionModal() {
        this.connectionModal?.destroy();
        this.connectionModal = await this.createModal("connection");
        this.modalData = {
            "oracleWalletUse": "N",
            "driverClass": "oracle.jdbc.driver.OracleDriver",
            "name": "MAIN CONNECTION",
            "language": "pt_br",
            "languageLabel": "Portuguese (BR)",
            "database": "Oracle",
            "main": true,
            "dialect": "org.hibernate.dialect.OracleDialect"
        };
        this.connectionModal.show();
    }
    public async saveConnectionModal() {
        const form = this.connectionModal.getChildInstance<Form>("connectionForm");
        if (!form.validate()) {
            return;
        }
        const formValues = form.value;
        const defaultData = this.modalData;
        const content = {
            changed: true,
            passwordNeedEncrypt: true,
            "dbUrl": formValues.dbUrl,
            "username": formValues.username,
            "password": formValues.password,
            "service": formValues.service,
            ...defaultData
        };

        try {
            const res = await Http.post("connection/save", {
                "requestType": "DataSet",
                "disableLoader": false,
                "dataset": [content]
            });
            CustomAlertService.saveSuccessAlert();
            this.openImportModal();
        } catch (error) {
            console.error(error);
            CustomAlertService.errorAlert("Ocorreu um erro.");
        }
        this.connectionModal?.hide();
    }
    public async testConnection() {
        const form = this.connectionModal.getChildInstance<Form>("connectionForm");
        if (!form.validate()) {
            return;
        }
        const formValues = form.value;
        const defaultData = this.modalData;
        const content = {
            changed: true,
            passwordNeedEncrypt: true,
            "dbUrl": formValues.dbUrl,
            "username": formValues.username,
            "password": formValues.password,
            "service": formValues.service,
            ...defaultData
        };
        let status = false;
        LoadingService.show("Aguarde...");
        try {
            const res = await Http.post("testConnection", {
                "requestType": "Row",
                "row": content
            });
            const data = res.data.dataset.testResponse;
            status = !data.hasError;
            if (!status) {
                this.showErrorDialog("Ocorreu um erro", data.response);
            } else {
                this.showSuccessDialog("Sucesso", "Conectado com sucesso" + (data.response.version ? " com o coletor na versão " + data.response.version : "!"));
            }
        } catch (error) {
            console.error(error);
            status = false;
            CustomAlertService.errorAlert("Oocrreu um erro.");
        }
        LoadingService.hide();
    }
    public closeConnectionModal() {
        this.connectionModal?.hide();
    }
}
